# Generated by roxygen2: do not edit by hand

export(comm_manager)
export(installspec)
export(jupyter_option_defaults)
export(log_debug)
export(log_error)
export(log_info)
export(main)
exportClasses(Comm)
exportClasses(CommManager)
import(digest)
import(methods)
import(uuid)
importFrom(IRdisplay,prepare_mimebundle)
importFrom(IRdisplay,publish_mimebundle)
importFrom(crayon,blue)
importFrom(crayon,green)
importFrom(crayon,red)
importFrom(evaluate,evaluate)
importFrom(evaluate,flush_console)
importFrom(evaluate,new_output_handler)
importFrom(evaluate,parse_all)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(pbdZMQ,ZMQ.MC)
importFrom(pbdZMQ,ZMQ.PO)
importFrom(pbdZMQ,ZMQ.SO)
importFrom(pbdZMQ,ZMQ.ST)
importFrom(pbdZMQ,zmq.bind)
importFrom(pbdZMQ,zmq.ctx.new)
importFrom(pbdZMQ,zmq.getsockopt)
importFrom(pbdZMQ,zmq.msg.recv)
importFrom(pbdZMQ,zmq.msg.send)
importFrom(pbdZMQ,zmq.poll)
importFrom(pbdZMQ,zmq.poll.get.revents)
importFrom(pbdZMQ,zmq.recv)
importFrom(pbdZMQ,zmq.recv.multipart)
importFrom(pbdZMQ,zmq.send)
importFrom(pbdZMQ,zmq.send.multipart)
importFrom(pbdZMQ,zmq.setsockopt)
importFrom(pbdZMQ,zmq.socket)
importFrom(repr,mime2repr)
importFrom(repr,repr_option_defaults)
importFrom(stats,setNames)
importFrom(utils,capture.output)
importFrom(utils,getFromNamespace)
importFrom(utils,getS3method)
importFrom(utils,head)
importFrom(utils,str)
importFrom(utils,tail)
