% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comm_manager.r
\docType{class}
\name{CommManager-class}
\alias{CommManager-class}
\alias{CommManager}
\title{The CommManager}
\description{
Has methods able to register comms/targets and process comm messages
}

